//******************************************************************************
//*
//*     FULLNAME:  Single-Chip Microcontroller Real-Time Operating System
//*
//*     NICKNAME:  scmRTOS
//*
//*     PROCESSOR: 8051
//*
//*     TOOLKIT:   EW8051 (IAR Systems)
//*               
//*     PURPOSE:   Target Dependent Stuff Source
//*               
//*     Version: 4.00
//*
//*     $Revision: 528 $
//*     $Date:: 2012-04-04 #$
//*
//*     Copyright (c) 2003-2012, Harry E. Zhurov
//*
//*     Permission is hereby granted, free of charge, to any person 
//*     obtaining  a copy of this software and associated documentation 
//*     files (the "Software"), to deal in the Software without restriction, 
//*     including without limitation the rights to use, copy, modify, merge, 
//*     publish, distribute, sublicense, and/or sell copies of the Software, 
//*     and to permit persons to whom the Software is furnished to do so, 
//*     subject to the following conditions:
//*
//*     The above copyright notice and this permission notice shall be included 
//*     in all copies or substantial portions of the Software.
//*
//*     THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, 
//*     EXPRESS  OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF 
//*     MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. 
//*     IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY 
//*     CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, 
//*     TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH 
//*     THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
//*
//*     =================================================================
//*     See http://scmrtos.sourceforge.net for documentation, latest
//*     information, license and contact details.
//*     =================================================================
//*
//******************************************************************************
//*     8051/IAR port by a9d, Copyright (c) 2011-2012

#include <scmRTOS.h>

using namespace OS;

OS::TPrioMaskTable OS::PrioMaskTable;

OS::TSavedSP OS::SavedSP;



#if scmRTOS_PROCESS_RESTART_ENABLE == 1
stack_item_t*  RStack_point[PROCESS_COUNT+1];

#if scmRTOS_DEBUG_ENABLE == 1
stack_item_t*  RStack_pool_point[PROCESS_COUNT+1];
#endif

#endif

//------------------------------------------------------------------------------
void TBaseProcess::init_stack_frame( stack_item_t * Stack
                                   , stack_item_t * RStack
                                   , stack_item_t  rs_size 
                                   , uint8_t pr  
                                   , void (*exec)() 
                                #if scmRTOS_DEBUG_ENABLE == 1
                                   , stack_item_t * StackPool
                                   , stack_item_t * RStackPool
                                #endif
                                   )
{ 
     static uint8_t free_space=0xFF; //0x21-0xFF
     
     free_space-=rs_size;
          
     if(free_space<=0x21)
     {
       while(1); //"rstack overflow"
     }
     
      RStack = ( stack_item_t*)( HARDWARE_STACK_ADDRESS + free_space);  //stack point 
     
     #if scmRTOS_DEBUG_ENABLE == 1
      RStackPool=RStack+rs_size;
      
      this->RStackPool=RStackPool; 
     #endif 
     
      #if scmRTOS_PROCESS_RESTART_ENABLE == 1
         RStack_point[pr]=RStack;
         
         #if scmRTOS_DEBUG_ENABLE == 1
          RStack_pool_point[pr]=RStackPool;
         #endif 
      #endif          
         
         
      *(++RStack)=reinterpret_cast<uint16_t>(exec);
      *(++RStack)=reinterpret_cast<uint16_t>(exec)>>8;
    
    StackPointer = Stack;      

    *(--StackPointer) =0x80;    //IEN0 register
    StackPointer     -= 1+2;    //A+DPTR
    *(--StackPointer) = reinterpret_cast<uint16_t>(RStack);// RStack;  //SP  
     StackPointer-=1+8+VREG_COUNT+1+1;   //PSW+REG+VREG_COUNT+VB+B
     
     uint8_t _spxl=(uint16_t)(StackPointer-2);
     uint8_t _spxh=((uint16_t)(StackPointer-2)>>8);
     
    *(--StackPointer) = _spxl;   //SPX
    *(--StackPointer) = _spxh;
   
    

#if scmRTOS_DEBUG_ENABLE == 1
    //-----------------------------------------------------------------------
    //   Fill stack pools with predefined value for stack consumption checking
    //   Do not fill area for GPR and SFR on data stack
    stack_item_t *fill_ptr;
    fill_ptr = RStack;
    while( fill_ptr < RStackPool )
    {
	*++fill_ptr = STACK_DEFAULT_PATTERN;
    }
    fill_ptr = StackPointer;
    while( fill_ptr > StackPool )
    {
	*--fill_ptr = STACK_DEFAULT_PATTERN;
    }
#endif // scmRTOS_DEBUG_ENABLE
}


#if scmRTOS_PROCESS_RESTART_ENABLE == 1
//------------------------------------------------------------------------------
void TBaseProcess::reinit_stack_frame( stack_item_t * Stack
                                   , stack_item_t * RStack
                                   , uint8_t pr  
                                   , void (*exec)() 
                                #if scmRTOS_DEBUG_ENABLE == 1
                                   , stack_item_t * StackPool
                                   , stack_item_t * RStackPool
                                #endif
                                   )
{               
      RStack=RStack_point[pr];
      
      #if scmRTOS_DEBUG_ENABLE == 1
          RStackPool=RStack_pool_point[pr];
      #endif 
     
      *(++RStack)=reinterpret_cast<uint16_t>(exec);
      *(++RStack)=reinterpret_cast<uint16_t>(exec)>>8;
    
    StackPointer = Stack;      

    *(--StackPointer) =0x80;    //IEN0 register
    StackPointer     -= 1+2;             //A+DPTR
    *(--StackPointer) = reinterpret_cast<uint16_t>(RStack);// RStack;          //SP  
     StackPointer-=1+8+VREG_COUNT+1+1;   //PSW+REG+VREG_COUNT+VB+B
     
     uint8_t _spxl=(uint16_t)(StackPointer-2);
     uint8_t _spxh=((uint16_t)(StackPointer-2)>>8);
     
    *(--StackPointer) = _spxl;   //SPX
    *(--StackPointer) = _spxh;
   
    

#if scmRTOS_DEBUG_ENABLE == 1
    //-----------------------------------------------------------------------
    //   Fill stack pools with predefined value for stack consumption checking
    //   Do not fill area for GPR and SFR on data stack
    stack_item_t *fill_ptr;
    fill_ptr = RStack;
    while( fill_ptr > RStackPool )
    {
	*--fill_ptr = STACK_DEFAULT_PATTERN;
    }
    fill_ptr = StackPointer;
    while( fill_ptr > StackPool )
    {
	*--fill_ptr = STACK_DEFAULT_PATTERN;
    }
#endif // scmRTOS_DEBUG_ENABLE
}
#endif //scmRTOS_PROCESS_RESTART_ENABLE

//------------------------------------------------------------------------------
#pragma vector=T4_VECTOR
OS_INTERRUPT void OS_SystemTimer_ISR(void)
{ 
    /* Clears the CPU interrupt flag. */
    T4IF = 0;  
    /* Clears the module interrupt flag. */
    T4OVFIF = 0;   
    
    scmRTOS_ISRW_TYPE ISR;  
    
    Kernel.system_timer();
    
   
    #if scmRTOS_SYSTIMER_NEST_INTS_ENABLE == 1
        //ABLE_NESTED_INTERRUPTS();
    #else 
        TCritSect cs;
    #endif

    #if scmRTOS_SYSTIMER_HOOK_ENABLE == 1
    system_timer_user_hook();
    #endif   
}
